import { Component, OnInit } from '@angular/core';
import {ActivatedRoute, Router} from '@angular/router';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {GameStatus} from '../interface/gameStatus';
import {io} from 'socket.io-client';
import {environment} from '../../environments/environment';
import {IsAllowedToPlay} from '../dao/isAllowedToPlay';

@Component({
  selector: 'app-overview',
  templateUrl: './overview.component.html',
  styleUrls: ['./overview.component.scss']
})
export class OverviewComponent implements OnInit {

  wantToBeExplainer = false;
  isActiveRound = false;
  socket: any;
  red = 'choose';
  blue = 'choose';
  nextTeam = 'Blau';
  team = '';
  buttonNewGameDisabled = true;
  buttonNextRoundDisabled = true;
  sessionName = '';
  membership = 'Wähle dein Team';
  gameStatus: GameStatus = {
    sessionID: -1,
    red: 0,
    blue: 0,
    redTurn: true,
    activeExplainer: 1,
    activeWatchdog: 1
  };
  constructor(private router: Router,
              private activatedRoute: ActivatedRoute,
              private service: TabuMiddlewareService,
              private isAllowedToPlay: IsAllowedToPlay) { }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      this.sessionName = String(params.get('sessionName')).toLowerCase();
      this.team = String(params.get('team'));
      console.log('Team:', this.team);
      if (this.team === 'red'){
        this.membership = 'Du gehörst zu Team rot!';
        this.buttonNewGameDisabled = false;
        this.red = 'red';
        this.blue = 'choose';
      }
      else if (this.team === 'blue'){
        this.membership = 'Du gehörst zu Team blau!';
        this.buttonNewGameDisabled = false;
        this.blue = 'blue';
        this.red = 'choose';
      }
      else if (this.team === 'null'){
        this.membership = 'Wähle dein Team';
      }
      else {
        this.socket.disconnect();
        this.router.navigate(['error']);
        return;
      }
      this.getGameStatus();
      this.socket = io(environment.tabuServerURL);
      this.listen();
    });
  }

  private listen(): any {
    this.socket.on(this.sessionName + ':newRound', (data: any) => {
      if (JSON.parse(data)) {
        console.log('OVERVIEW: ', data);
        if (((this.team === 'red' && this.nextTeam === 'Rot') || (this.team === 'blue' && this.nextTeam === 'Blau'))
          && !this.wantToBeExplainer) {
          console.log('OVERVIEW GO GUESSER: ', data);
          this.socket.disconnect();
          this.isAllowedToPlay.isAllowed = true;
          this.isAllowedToPlay.role = 'guesser';
          this.router.navigate([this.sessionName + '/' + this.team + '/guesser']);
        } else if ((this.team === 'red' || this.team === 'blue') && !this.wantToBeExplainer) {
          console.log('OVERVIEW GO WATCHDOG: ', data);
          this.socket.disconnect();
          this.isAllowedToPlay.isAllowed = true;
          this.isAllowedToPlay.role = 'watchdog';
          this.router.navigate([this.sessionName + '/' + this.team + '/watchdog']);
        }
      }
    });
    this.socket.on(this.sessionName + ':endRound', (data: any) => {
      if (JSON.parse(data)) {
        this.getGameStatus();
      }
    });
    this.socket.on(this.sessionName + ':newCard', () => {
      this.getGameStatus();
    });
    this.socket.on(this.sessionName + ':newGame', () => {
      this.getGameStatus();
    });
  }

  startGame(): void {
    if (this.team === 'red' || this.team === 'blue') {
      this.service.newGame({spielname: this.sessionName}).then(value => {
        const status = JSON.parse(value.status);
        if (status) {
          this.getGameStatus();
        }
      }).catch(reason => console.log(reason));
    }
  }

  nextRound(): void {
    this.wantToBeExplainer = true;
    this.service.newRound({spielname: this.sessionName});
    this.socket.disconnect();
    this.isAllowedToPlay.isAllowed = true;
    this.isAllowedToPlay.role = 'explainer';
    this.router.navigate([this.router.url + '/explainer', ]);
  }

  joinTeam(team: string): void {
    this.team = team;
    this.socket.disconnect();
    this.router.navigate([this.sessionName + '/' + team]);
  }

  getGameStatus(): void {
    this.service.getGamestatus({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.gameStatus.sessionID = JSON.parse(value.sessionID);
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
        this.gameStatus.redTurn = JSON.parse(value.redTurn);
        this.gameStatus.activeExplainer = JSON.parse(value.activeExplainer);
        this.gameStatus.activeWatchdog = JSON.parse(value.activeWatchdog);
        if (this.gameStatus.redTurn){
          this.nextTeam = 'Rot';
          this.buttonNextRoundDisabled = this.team !== 'red';
        }
        else{
          this.nextTeam = 'Blau';
          this.buttonNextRoundDisabled = this.team !== 'blue';
        }
        this.isActiveRound = this.gameStatus.activeExplainer === 1;
      }
      console.log(this.gameStatus);
    }).catch(reason => console.log(reason));
  }
}
